#!/usr/bin/env sh

#
# Copyright(c) 2006-2021 ThreatSTOP, Inc. All Rights Reserved
#
# NOTICE: All information contained herein is, and remains the property
# of ThreatSTOP, Inc. and its suppliers, if any. The intellectual and
# technical concepts contained herein are proprietary to ThreatSTOP, Inc.
# and its suppliers and certain aspects thereof are protected by United
# States Patent No. 8,533,822 and United States Patent No. 8,869,237,
# and are protected by trade secret or copyright law.
#
# US Government Entities: The ThreatSTOP service, software and
# documentation, as applicable, are "commercial computer software" and
# "commercial computer software documentation" developed exclusively at
# private expense by Threatstop, Inc. ("Threatstop"). Pursuant to FAR
# 12.212 or DFARS 227 7202 and their successors, as applicable, use,
# reproduction and disclosure of the software and documentation is
# governed by the terms of Threatstop's commercial agreement.
#

#
# ThreatSTOP OpenBSD PF Install Script
# This script copies the scripts to /usr/local/sbin and the configuration file
# to /usr/local/etc. If the files exist, you will be prompted on whether you
# to replace the files. It is recommended that you do not overwrite the
# configuration file and merge the threatstop.conf.example file with your
# current file.
#
# The script also creates 2 cron jobs. The first is running the script that
# updates the block lists, threatstop-pf.sh. This will run every 2 hours. The
# second uploads the log file to ThreatSTOP. This runs every night at 1:00 AM.
# In order to prevent duplicate log entries, the log file needs to be rotated
# at least once a day. This script updates the pflog entry in /etc/newsyslog.conf
# to rotate the log ever night at 12:00 AM and removes the size check.
#
###############################################################################

PREFIX="/usr/local"

# Check to make sure script is run by root
if [ `id -u` != "0" ]
then
    echo "You must run the script as root. Exiting"
    exit 1
fi

# Make sure the directories exist. Create if needed.
if [ ! -d $PREFIX/sbin ]
then
    echo "$PREFIX/sbin does not exist. Creating."
    mkdir -p $PREFIX/sbin
fi

if [ ! -d $PREFIX/etc ]
then
    echo "$PREFIX/etc does not exist. Creating"
    mkdir -p $PREFIX/etc
fi

if [ ! -f /etc/pf.conf ];then
    echo "We did not detect /etc/pf.conf exists. You will need to create a basic PF configuration (/etc/pf.conf) and reboot before adding table configuration."
    exit 2
fi

if whereis sysrc; then
    # check sysrc if pf enabled
    echo "checking if pf is enabled"
    if sysrc pf_enable | awk '{print $2}' | grep -q -i 'no';then
        printf 'PF is not enabled. Do you want to enable this now? [y/N]'; read ANS
        if [ "[$ANS]" = "[y]" ];then
                echo "Enabling PF"
                sysrc pf_enable="YES"
        else
                echo "You will need to run 'sysrc pf_enable=\"YES\"' to enable pf before we can use pf."
                exit 71
        fi
    fi
    # check sysrc if pflog enabled
    echo "checking if pflog is enabled"
    if sysrc pflog_enable | awk '{print $2}' | grep -q -i 'no';then
        printf 'PFLOG is not enabled. Do you want to enable this now? [y/N]'; read ANS
        if [ "[$ANS]" = "[y]" ];then
            echo "Enabling PFLOG"
            sysrc pflog_enable="YES"
        else
            echo "You will need to run 'sysrc pf_enable=\"YES\"' to enable pf before you can export logs."
        fi
    fi
fi

# if FreeBSD tell end-user to ensure dig is installed
if ! dig -v; then
    echo "You do not have 'dig' (a DNS lookup utility) installed."
    if uname -s | grep -q -i 'freebsd';then
            printf 'Do you want to install this now? [y/N]'; read ANS
            if [ "[$ANS]" = "[y]" ];then
                echo "installing dns/bind-tools"
                pkg install dns/bind-tools
            else
                echo "You will need to install dig (dns/bind-tools) by running 'pkg install dns/bind-tools' enable policy updates."
                exit 86
            fi
    else
            echo "Check your distribution software sources to see how you install it. It is required for this integration. exiting..."
            exit 90
    fi

fi

# install perl libraries
if ! perl -e 'use strict; use LWP::UserAgent; use HTTP::Request::Common; use LWP::Protocol::https;';then
    if uname -s | grep -q -i 'freebsd';then
            echo "To enable logging we need to install [p5-libwww]."
            printf 'Do you want to install this now? [y/N]'; read ANS
            if [ "[$ANS]" = "[y]" ];then
                echo "installing p5-libwww p5-LWP-Protocol-https"
                pkg install p5-libwww p5-LWP-Protocol-https
            else
                echo "You will need to install p5-libwww by running 'pkg install p5-libwww' enable log uploads."
            fi
    elif uname -s | grep -q -i 'openbsd';then
        echo "To enable logging we need to install [p5-libwww p5-LWP-Protocol-https wget]."
        printf 'Do you want to install this now? [y/N]'; read ANS
        if [ "[$ANS]" = "[y]" ];then
            echo "installing p5-libwww p5-LWP-Protocol-https ..."
            export PKG_PATH=http://ftp5.usa.openbsd.org/pub/OpenBSD/`uname -r`/packages/`machine -a`/
            pkg_add p5-libwww p5-LWP-Protocol-https
        else
            echo "You will need to install [p5-libwww p5-LWP-Protocol-https wget] to enable log uploads."
        fi
    fi
fi

# Copy the scripts to $PREFIX/sbin
echo "Copying scripts to \"$PREFIX/sbin\""
cp -i threatstop-pf.sh sendlog.sh loguploadclient.pl ptr.sed apl.sed $PREFIX/sbin

# Copy the configuration file to $PREFIX/etc
echo;echo "Copying configuration and version file to \"$PREFIX/etc\""
cp -i threatstop.conf $PREFIX/etc
cp -i VERSION $PREFIX/etc/threatstop.VERSION

# Create the cron jobs
MIN=`date +%M`
# Append new jobs to the working file
if crontab -l | grep -q 'ThreatSTOP' ;then
    echo "Crontab already contains ThreatSTOP configuration... [Skipping]"
else

    echo;echo "Adding crontab entry to run the \"threatstop-pf.sh\" script every hour"
    echo "and upload the log file with the \"sendlog.sh\" script every 15mins"
    echo "  Backing up original crontab to \"crontab.pre-threatstop\""
    crontab -l > crontab.pre-threatstop   # Make a copy
    cp crontab.pre-threatstop newcrontab  # Create working file from the backup
    cat >> newcrontab <<CRONTAB

# ThreatSTOP update
$MIN      *     *       *       *       /usr/local/sbin/threatstop-pf.sh > /var/log/threatstop_policy_updates.log
*/15       *       *       *       *       /usr/local/sbin/sendlog.sh > /var/log/threatstop_sendlogs.log
CRONTAB

  crontab newcrontab    # Replace crontab with new file
  rm newcrontab         # Delete the working file
fi

# Change newsyslog to rotate the pflog file at midnight
echo;echo "Changing the newsyslog.conf file to rotate pflog every hour"
if [ ! -f /etc/newsyslog.conf.d/pf.conf.pre-threatstop ] && [ ! -f /etc/newsyslog.conf.pre-threatstop ];then
    if [ -f /etc/newsyslog.conf.d/pf.conf ];then
        echo "  Creating backup of /etc/newsyslog.conf.d/pf.conf => /etc/newsyslog.conf.d/pf.conf.pre-threatstop"
        cp /etc/newsyslog.conf.d/pf.conf /etc/newsyslog.conf.d/pf.conf.pre-threatstop
        sed '/pflog/ s/[0-9].*[ ]*[JZ]B/600    10  \*   1   ZB/' < /etc/newsyslog.conf.d/pf.conf.pre-threatstop > /etc/newsyslog.conf.d/pf.conf
        printf "Testing the newsyslog config.........."
        if newsyslog -v -f /etc/newsyslog.conf.d/pf.conf > /tmp/newsyslogout 2>&1;then
            echo "[ok]"
        else
            echo "[fail]"
            cat /tmp/newsyslogout
        fi
  else
      echo "  Creating backup of /etc/newsyslog.conf => /etc/newsyslog.conf.pre-threatstop"
      cp /etc/newsyslog.conf /etc/newsyslog.conf.pre-threatstop
      sed '/pflog/ s/[0-9].*[ ]*[JZ]B/600    10  \*   1   ZB/' < /etc/newsyslog.conf.pre-threatstop > /etc/newsyslog.conf
      printf "Testing the newsyslog config.........."
        if newsyslog -v -f /etc/newsyslog.conf > /tmp/newsyslogout 2>&1;then
            echo "[ok]"
        else
            echo "[fail]"
            cat /tmp/newsyslogout
        fi
  fi
else
   echo "No newsyslog conf update needed."
fi

echo "Install complete at `date`"
exit 0