#!/usr/bin/env sh
# set -x
#
# Copyright(c) 2006-2021 ThreatSTOP, Inc. All Rights Reserved
#
# NOTICE: All information contained herein is, and remains the property
# of ThreatSTOP, Inc. and its suppliers, if any. The intellectual and 
# technical concepts contained herein are proprietary to ThreatSTOP, Inc. 
# and its suppliers and certain aspects thereof are protected by United 
# States Patent No. 8,533,822 and United States Patent No. 8,869,237, 
# and are protected by trade secret or copyright law.
#
# US Government Entities: The ThreatSTOP service, software and
# documentation, as applicable, are "commercial computer software" and
# "commercial computer software documentation" developed exclusively at
# private expense by Threatstop, Inc. ("Threatstop"). Pursuant to FAR
# 12.212 or DFARS 227 7202 and their successors, as applicable, use,
# reproduction and disclosure of the software and documentation is
# governed by the terms of Threatstop's commercial agreement.
#

#
# ThreatSTOP OpenBSD PF List Update Script
# These scripts get your ThreatSTOP block list and creates files that PF can
# use as tables. Please see the README.txt file for details on running the 
# scripts.
#
###############################################################################


PREFIX="/usr/local"
CONFIG="$PREFIX/etc/threatstop.conf"
SED_PTR="$PREFIX/sbin/ptr.sed"
SED_APL="$PREFIX/sbin/apl.sed"
VERSION=$(cat $PREFIX/etc/threatstop.VERSION)
LINE="============================================================"

EXIT_STATUS=0


echo "Starting ThreatSTOP v$VERSION update on `date`" ; echo
START=`date +%R:%S`
START_EPOCH=`date +%s`


echoerr() { printf "`date +'%m-%d %H:%M:%S'` %s\n" "$*" >&2; }


tstamp_print() {
	echo "`date +'%m-%d %H:%M:%S'` $*"
}

tstamp_nprint() {
	echo -n "`date +'%m-%d %H:%M:%S'` $*"
}

# Check for and read the config file, abort if not found
if [ ! -f $CONFIG ]
then
	echo "CRITICAL: Cannot read config file: \"$CONFIG\""
	echo "Exiting"
	exit 1
else
	. $CONFIG
fi

# Create temporary files. These are deleted at the end of the script
BLOCK_TMP=`mktemp /tmp/tsblocktmp.XXXXXXXXXX`
ALLOW_TMP=`mktemp /tmp/tsallowtmp.XXXXXXXXXX`

LISTS="$ALLOW_LIST $BLOCK_LIST" 

# Check the availability of the ThreatSTOP DNS servers. Use the first one
# that works, abort if none of them work.
for DNS in $DNS_SERVERS
do
	RUN=`dig @$DNS basic.threatstop.local PTR | sed -n -f $SED_PTR |wc -l`
	if [ $RUN -gt 0 ]
	then
		OKDNSLIST="$OKDNSLIST $DNS"
		#echo "DNS Server $DNS worked"
	else
		echoerr "DNS Server $DNS failed"
	fi
done
if [ `echo $OKDNSLIST | wc -w` -eq 0 ]
then
	echoerr "No ThreatSTOP DNS servers are reachable. Aborting"
	exit 1
fi
set $OKDNSLIST
DNS=$1
echo "Using DNS Server $DNS"


# Do a PTR lookup on the main list to get the full complement of 
# block and/or allow lists. Save to temporary files.
DATE=`date`
process_list() {
	LISTTYPE=$1
	LIST=$2
	TMPFILE=$3
	tstamp_print "Processing $LISTTYPE list..."
	echo "# $LISTTYPE list $LIST generated on $DATE" >> $TMPFILE
	for PTR in `dig @$DNS $LIST PTR | sed -n -f $SED_PTR`
	do
		tstamp_print "Processing $LISTTYPE list $PTR"
		dig @$DNS $PTR -t APL +tcp | sed -n -f $SED_APL | sh >> $TMPFILE
	done
}

if [ ! -z $ALLOW_LIST ];then
	process_list allow $ALLOW_LIST $ALLOW_TMP
	# WARN end-user if no table reference found in /etc/pf.conf
	if ! grep -q $ALLOW_FILE /etc/pf.conf;then
		echoerr "** WARNING **: $0 did not locate a reference to '$ALLOW_FILE' in /etc/pf.conf. The table may not be properly loading if this is not corrected."
	fi
fi
if [ ! -z $BLOCK_LIST ];then
	process_list block $BLOCK_LIST $BLOCK_TMP
	# WARN end-user if no table reference found in /etc/pf.conf
	if ! grep -q $BLOCK_FILE /etc/pf.conf;then
		echoerr "** WARNING **: $0 did not locate a reference to '$BLOCK_FILE' in /etc/pf.conf. The table may not be properly loading if this is not corrected."
	fi
fi

echo ; echo "Completed getting all the lists"

# Copy the results to their final location
BCNT=0
BCNT=`sed -n '$=' $BLOCK_TMP`
if [ $BLOCK_LIST ] && [ $BCNT -gt 0 ]
then
	BCNT=`expr $BCNT - 1`
	tstamp_nprint "Adding $BCNT blocked addresses "
	# cp $BLOCK_TMP $BLOCK_FILE
	# sort and remove duplicates
	if sort -u -t . -k 1,1n -k 2,2n -k 3,3n -k 4,4n $BLOCK_TMP > $BLOCK_FILE;then
		echo '[OK]'
	else
		echo '[FAIL]'
		echoerr "$0 failed processing 'block' addresses"
	fi
fi

ACNT=0
ACNT=`sed -n '$=' $ALLOW_TMP`
if [ $ALLOW_LIST ] && [ $ACNT -gt 0 ]
then
	ACNT=`expr $ACNT - 1`
	tstamp_nprint "Adding $ACNT allowed addresses "
	# cp $ALLOW_TMP $ALLOW_FILE
	# sort and remove duplicates
	if sort -u -t . -k 1,1n -k 2,2n -k 3,3n -k 4,4n $ALLOW_TMP > $ALLOW_FILE;then
		echo '[OK]'
	else
		echo '[FAIL]'
		echoerr "$0 failed processing 'allow' addresses"
	fi
fi


# Check if we're over limit, truncate if so
MAX_LIMIT=`sysctl net.pf.request_maxcount 2>/dev/null| awk '{print $2}'`
TOTAL_CNT=`expr $ACNT + $BCNT`
if [ ! -z $MAX_LIMIT ] && [ 0${TOTAL_CNT} -gt $MAX_LIMIT ];then
	echoerr "** WARNING ** The updated policy downloaded a combined size of $TOTAL_CNT which exceeds the max policy size set by net.pf.request_maxcount [$MAX_LIMIT]. Addresses past this limit will not be loaded. "
	LINES_TO_REMOVE=`expr $TOTAL_CNT - $MAX_LIMIT`
	tstamp_print "We will have to remove $LINES_TO_REMOVE addresses"
	LINES_TO_REMOVE=`expr $BCNT - $LINES_TO_REMOVE`
	tstamp_print "$LINES_TO_REMOVE addresses remaining in block list"
	sed -i.untruncated "${LINES_TO_REMOVE},\$ d" $BLOCK_FILE
	EXIT_STATUS=3
fi

# Delete the temporary files
rm $BLOCK_TMP $ALLOW_TMP




# Reload PF, if it is enabled in the configuration file.
if [ $RELOAD_PF == "YES" ]
then


	NEEDS_PF_CONF_RELOAD=0
	# If there is an allow list in use, we can reload both the
	# block and allow tables with one command.
	#if [ $ACNT -gt 0 ]
	PFCTL_TBL=''
	if [ ! -z $ALLOW_LIST ];then
		if ! pfctl -t ${ALLOW_TABLE} -T show > /dev/null 2>&1;then
			NEEDS_PF_CONF_RELOAD=1
			tstamp_print "$ALLOW_TABLE was not found, it must be created"
		fi
		PFCTL_TBL="${PFCTL_TBL} -t $ALLOW_TABLE"
	fi
	if [ ! -z $BLOCK_LIST ];then
		if ! pfctl -t ${BLOCK_TABLE} -T show > /dev/null 2>&1;then
			NEEDS_PF_CONF_RELOAD=1
			tstamp_print "$BLOCK_TABLE was not found, it must be created"
		fi
		PFCTL_TBL="${PFCTL_TBL} -t $BLOCK_TABLE"
	fi

	if uname -s | grep -q FreeBSD;then
		NEEDS_PF_CONF_RELOAD=1
	fi



	if [ $NEEDS_PF_CONF_RELOAD -eq 1 ];then

		if uname -s | grep -q FreeBSD;then
			tstamp_nprint "Reloading pf on FreeBSD..."
			if /sbin/pfctl -f /etc/pf.conf $PFCTL_TBL;then
				echo "[OK]"
			else
				echo "[FAIL]"
				EXIT_STATUS=2
			fi
			echo "done"
		else
			tstamp_nprint "Loading OpenBSD pf tables"
			if /sbin/pfctl -f /etc/pf.conf;then
				echo "[OK]"
			else
				echo "[FAIL]"
				EXIT_STATUS=2
			fi
		fi
	fi

	# OpenBSD can just replace table contents if they exist
	if uname -s | grep -q OpenBSD;then
		tstamp_print "Replacing pf table(s) on OpenBSD..."

		# Replace table contents
		if [ ! -z $ALLOW_LIST ];then
			tstamp_nprint "Replacing Allow Table: $ALLOW_TABLE "
			if /sbin/pfctl -t ${ALLOW_TABLE} -T replace -f ${ALLOW_FILE};then
				echo "[OK]"
			else
				echo "[FAIL]"
				EXIT_STATUS=2
			fi
		fi
		if [ ! -z $BLOCK_LIST ];then
			tstamp_nprint "Replacing Block Table: $BLOCK_TABLE "
			if /sbin/pfctl -t ${BLOCK_TABLE} -T replace -f ${BLOCK_FILE};then
				echo "[OK]"
			else
				echo "[FAIL]"
				EXIT_STATUS=2
			fi
		fi

		echo "done"
	fi
fi

# Calculate runtime
END_EPOCH=`date +%s`
SECONDS=$((END_EPOCH - START_EPOCH))
HOURS=$((SECONDS / 3600))
SECONDS=$((SECONDS % 3600))
MINUTES=$((SECONDS / 60))
SECONDS=$((SECONDS % 60))

echo ; echo "Finished ThreatSTOP update at `date`"
echo "Run Length: $HOURS hour(s) $MINUTES minute(s) $SECONDS second(s)"
exit $EXIT_STATUS
